#!/bin/bash

#===============================================================================
#
#      FILE:  transfer.sh
# 
#      USAGE:  bash transfer.sh $1 (example: bash transfer.sh validation) 
# 
#      DESCRIPTION:  
# 
#      ARGUMENTS: transfer, validation, both, no argument defaults to both
#
#      NOTES: Source should be one level deeper then the destination, and both should have a trailing /
#
#      VERSION:  1
#===============================================================================

SOURCE="/appl/local/climatedt/input_data/applications/" # Source gets a trailing / in the rysnc line anyway
DESTINATION="transfer1:/gpfs/scratch/ehpc01/input_data/" # Should always have a trailing / and be one level higher then source

# Log files
SOURCE_NAME=$(basename "$SOURCE") # Extract source folder name for log file naming
LOG_DIR="/users/hearnesh/data_admin/transfer_logs"
TIMESTAMP=$(date +'%Y%m%d_%H%M%S')
LOG_FILE="$LOG_DIR/${SOURCE_NAME}_t_log_$TIMESTAMP.txt"
ERROR_LOG_FILE="$LOG_DIR/${SOURCE_NAME}_t_err_log_$TIMESTAMP.txt"
VALIDATION_LOG_FILE="$LOG_DIR/${SOURCE_NAME}_val_log_$TIMESTAMP.txt"

# Move to the parent directory of the source (to prevent full path being transferred)
cd "$(dirname "$SOURCE")"

transfer() {
    # Perform the transfer and log it
    echo "Starting rsync transfer..."
    time rsync -aPR --stats --delete "$(basename "$SOURCE")/" "$DESTINATION" > "$LOG_FILE" 2>> "$ERROR_LOG_FILE" &
    RSYNC_PID=$!
    
    echo "Transfer started in background. Check logs at:"
    echo "  - Progress: $LOG_FILE"
    echo "  - Errors:   $ERROR_LOG_FILE"
    echo "Use 'tail -f $LOG_FILE' to monitor."

    # Wait for the rsync process to finish
    wait "$RSYNC_PID"

    echo "Transfer complete at $(date)."
}

validation() {
    # Perform rsync validation. This does not transfer any files, it just uses checksums
    echo "Starting rsync validation..."
    nohup rsync -aPRc --dry-run --delete "$(basename "$SOURCE")/" "$DESTINATION" &>> "$VALIDATION_LOG_FILE"  # Both output and errors go to the same log file

    echo "Validation complete. Check $VALIDATION_LOG_FILE for details."
}

# Define argument for action: transfer+validation or validation only
ACTION="${1:-both}"  # Default to 'both' if no action is passed (runs transfer + validation)

# Perform the action based on the argument passed
case "$ACTION" in
    transfer)
	echo "Transfer only"
        transfer  
        ;;
    validation)
        echo "Validation only"
	validation 
        ;;
    both)
        echo "Transfer and then validation"
	transfer 
        validation 
        ;;
    *)
        # Invalid action
        echo "Usage: $0 {transfer|validation|both}"
        exit 1
        ;;
esac
